#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Crée le Wed Feb 16 19:59:00 2022

@auteur: David ALBERTO
(www.astrolabe-science.fr)
"""
import matplotlib.pyplot as plt
import matplotlib as mpl#Normalisation des valeurs
import matplotlib.colorbar as cbar#barres de couleurs
import numpy as np#pour remplacer les valeurs absentes par 'NaN' (Not a Number)
from mpl_toolkits.axes_grid1 import make_axes_locatable#positionner les barres de couleurs
from mendeleev import element

Zmax=108#on peut limiter aux premiers éléments chimiques
nom_grandeur='ConductiviteThermique'# pour accoler au nom du fichier (voir dernières lignes)
"""
Les données atomiques sont à choisir parmi (liste non exhaustive) :
abundance_crust (mg/kg)
abundance_sea (mg/L)
atomic_volume
boiling_point
melting_point
evaporation_heat
fusion_heat
heat_of_formation
ionenergy
is_radioactive (booléen)
lattice_constant
metallic_radius
specific_heat
thermal_conductivity (W/(mK))
vdw_radius

"""
symbole=[]
grandeur=[]
for N in range(1,Zmax+1):
    symbole.append(element(N).symbol)
    # valeur=element(N).electronegativity('pauling') # CHOIX DE LA GRANDEUR 
    # valeur=element(N).ionenergies[1]#énergie d'ionisation (eV)(Zmax=108)
    valeur=element(N).thermal_conductivity
    if valeur is None:
        valeur=np.nan
    grandeur.append(valeur)

# Réglages d'écriture du symbole des éléments :
taille_symbole=16
c_symbole='k'#couleur symbole

# Réglages d'écriture du numéro atomique :
x_offset_Z=-0.4
y_offset_Z=0.33
taille_Z=6
c_Z='k'#couleur numéro atomique

# Réglages d'écriture des terres rares :
y_offset_La=2.5#décalage vertical lanthanides
y_offset_Ac=y_offset_La#décalage vertical actinides

# Réglages d'écriture du nom des éléments :
x_offset_nom=-0.0
y_offset_nom=-0.4
c_nom='black'
taille_nom=6

noms=['Hydrogène','Hélium','Lithium','Béryllium','Bore','Carbone','Azote','Oxygène',
      'Fluor','Néon','Sodium','Magnésium','Aluminium','Silicium','Phosphore','Soufre',
      'Chlore','Argon','Potassium','Calcium','Scandium','Titane','Vanadium','Chrome',
      'Manganèse','Fer','Cobalt','Nickel','Cuivre','Zinc','Gallium','Germanium',
      'Arsenic','Sélénium','Brome','Krypton','Rubidium','Strontium','Yttrium',
      'Zirconium','Niobium','Molybdène','Technétium','Ruthénium','Rhodium',
      'Palladium','Argent','Cadmium','Indium',
'Étain','Antimoine','Tellure','Iode','Xénon','Césium','Baryum','Lanthane','Cérium',
'Praséodyme','Néodyme','Prométhium','Samarium','Europium','Gadolinium','Terbium',
'Dysprosium','Holmium','Erbium','Thulium','Ytterbium','Lutetium','Hafnium','Tantale',
'Tungstène','Rhénium','Osmium','Iridium','Platine','Or','Mercure','Thallium','Plomb',
'Bismuth','Polonium','Astate','Radon','Francium','Radium','Actinium','Thorium','Protactinium',
'Uranium','Neptunium','Plutonium','Americium','Curium','Berkelium','Californium','Einsteinium',
'Fermium','Mendelevium','Nobelium','Lawrencium','Rutherfordium','Dubnium','Seaborgium','Bohrium',
'Hassium','Meitnerium','Darmstadtium','Roentgenium','Copernicium','Nihonium','Flerovium',
'Moscovium','Livermorium','Tennessine','Oganesson',
]#le module mendeleev n'a que les noms des éléments en anglais
# ---------------------------------

a=3#espace horizontal
b=3#espace vertical
#############################################
# FONCTION :
def numeroscolonnes():#numérote les colonnes en haut
    for i in np.arange(1, 19):
        plt.text(a*(i-0.2), -0.4, "%s"%(i), fontsize=8, color='gray')

#############################################

#création des listes de coordonnées des éléments
"""
La ligne est la valeur de la période de l'élément.
La colonne est la valeur du paramètre "group_id" de l'élément.
Pour les terres rares, le module ne fournit pas de valeur pour "group_id" ; 
je me base donc sur la valeur de N pour établir un numéro de colonne.
Les listes "ligne" et "colonne" serviront ensuite pour toutes les indications
(masse molaire, N° atomique, symbole, nom).
"""
ligne=[]
colonne=[]

for Z in range(1,Zmax+1):
    if Z>=58 and Z<=71:#lanthanides
        colonne.append((3+Z-57)*a)
        ligne.append((element(Z).period+y_offset_La)*b)
    elif Z>=90 and Z<=103:#actinides
        colonne.append((3+Z-89)*a)
        ligne.append((element(Z).period+y_offset_Ac)*b)
    else:
        colonne.append((element(Z).group_id)*a)
        ligne.append((element(Z).period)*b)
# ---------------------------------

# plt.rcParams["font.family"] = "Ubuntu"#ou autre police installée
#paramètres du graphique
fig=plt.figure(figsize=(29.7/2.54,21/2.54),tight_layout=True)
plt.rcParams.update({"text.usetex": True}) #utiliser la police LaTeX (optionnel)

"""
Attention, en modifiant les dimensions (figsize, et valeurs des paramètres a et b, les tailles des polices
devront sans doute être réajustées.
"""
plt.title(r"$\textsf{Conductivité thermique}$", fontsize=20,y=0.85)

ax1=plt.subplot(111)
ax1.set_aspect('equal')

plt.xlabel("Conductivité thermique (W/(mK))")
plt.xticks([])
plt.yticks([])
plt.ylim(-10.5*b, 1)
plt.xlim(0,19*a)

#gamme de couleurs choisie (colormap) :
cmap=plt.get_cmap('YlGn')
"""
Les colormaps sont à choisir parmi les séquentiels :
'Greys', 'Purples', 'Blues', 'Greens', 'Oranges', 'Reds',
'YlOrBr', 'YlOrRd', 'OrRd', 'PuRd', 'RdPu', 'BuPu',
'GnBu', 'PuBu', 'YlGnBu', 'PuBuGn', 'BuGn', 'YlGn'
"""
#normalisation de la gamme de valeurs :
norm = mpl.colors.Normalize(vmin=min(grandeur),vmax=max(grandeur)) 

# #tracé des cases :
for Z in range(1,Zmax+1):
    plt.text(colonne[Z-1], -ligne[Z-1], symbole[Z-1], fontsize=taille_symbole, va='center', ha='center')
    plt.text(colonne[Z-1]+x_offset_Z*a,-ligne[Z-1]+y_offset_Z*b,Z,c=c_Z,fontsize=taille_Z,ha='left')#numéro atomique
    plt.text(colonne[Z-1]+x_offset_nom*a,-ligne[Z-1]+y_offset_nom*b,r'\textsf{%s}'%(noms[Z-1]),c=c_nom,fontsize=taille_nom,ha='center')#nom
    if grandeur[Z-1] is np.nan:
        couleur_case='gainsboro'
    else:
        couleur_case=cmap(norm(grandeur[Z-1]))
    rectangle=plt.Rectangle((colonne[Z-1]-0.5*a,-(ligne[Z-1]+0.5*b)),a,b,facecolor=couleur_case,edgecolor='k')#case
    ax1.add_patch(rectangle)

numeroscolonnes()

#tracé de la barre de couleurs :
divider = make_axes_locatable(ax1)
cax = divider.append_axes('bottom', size='6%', pad=0.3)
cb=cbar.ColorbarBase(cax, cmap=cmap, norm=norm, orientation='horizontal')

#----------------------------------------------------------------------------------------

fig.savefig("TableauPeriodique"+nom_grandeur+".pdf")
fig.savefig("TableauPeriodique"+nom_grandeur+".png",dpi=300)
# plt.show()
